#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vs1000.h>
#include <audio.h>
#include <mappertiny.h>
#include <minifat.h>
#include <codec.h>
#include <vsNand.h>

#include <player.h>

#define AUDIO_TESTS /* enable audio tests: KEY_5 pressed at power-on */


#ifdef AUDIO_TESTS
const struct KeyMapping myEmptyMap[] = {
  {KEY_LONG_PRESS|KEY_POWER, ke_powerOff},
  {0, ke_null}
};
auto s_int16 Sine(register __a0 u_int16 ph);
s_int16 ISine(u_int16 ph) {
    if (ph & 7) {
	register u_int16 f = (ph << 13);
	return (s_int16)(((s_int32)Sine(ph) * (f^0xffffU) +
			  (s_int32)Sine(ph+8) * f) >> 16);
    }
    return Sine(ph);
}

void AudioTests(void) {
    extClock4KHz = 12288/4;
    clockX = 7; /* 3.5x */
    SetRate(48000U); /* actual rate 46875Hz */
    bassReg = earSpeakerReg = 0;
    volumeReg = 0;
    SetVolume();

    /* Fix digital volume */
#define AP_IMD
    {
	register u_int32 phs = 0, add = 0;
	__y u_int16 ph3, ph5, ph7;
	register s_int16 *p = (s_int16 *)minifatBuffer;
	u_int16 wait = 0, mode = 0, ph2, add2;
	while (1) {
	    if (keyCheck == KEY_3) {
		keyCheck = 0;
		add += 0x5d00000UL; /* ~1kHz step */
	    }
	    if (keyCheck == KEY_4) {
		keyCheck = 0;
		add -= 0x5d00000UL; /* ~1kHz step */
	    }
	    if (keyCheck == KEY_1) { /* vol down */
		keyCheck = 0;
		if (mode == 2) {
		    volumeReg ^= 0x00feU;
		    SetVolume();
		}
	    }
	    if (keyCheck == KEY_2) {
		keyCheck = 0;
		if (mode == 0) {
		    if (USEX(DAC_VOL) == 0xb0b0) {
			USEX(DAC_VOL) = 0x0000;
		    } else {
			USEX(DAC_VOL) += 0x1010;
		    }
		} else if (mode == 2) {
		    volumeReg ^= 0xfe00U;
		    SetVolume();
		}
	    }
	    if (keyCheck == KEY_POWER) {
		/* next test */
		keyCheck = 0;
		if (++mode > 3)
		    mode = 0;
		if (mode == 0) {
		    /* frequency sweep */
		    extClock4KHz = 12288/4;
		    clockX = 8; /* 4.0x */
		    SetRate(48000U);
		    bassReg = earSpeakerReg = 0;
		    volumeReg = 0;
		    SetVolume();
		}
		if (mode == 1) {
		    /* amplitude sweep */
		    extClock4KHz = 12000/4;
		    SetRate(44100U);
		    volumeReg = 0x8080;
		    SetVolume();
		    add = 0x05d00000UL;
		}
		if (mode == 2) {
		    /* sin / crosstalk */
		    volumeReg = 0x00fe;
		    add = 0x05d00000UL * 10; /* Start at 10kHz */
		    SetVolume();
		}
		if (mode == 3) {
		    /* IMD 0.75% 60Hz 0.25% 7kHz */
		    extClock4KHz = 12288/4;
		    SetRate(48000U);
		    volumeReg = 0x0000;
		    SetVolume();
#ifdef AP_IMD
		    /* 3kHz pulse, ~15kHz sin */
		    add2 = 4096;
		    //add  = 20490L*65536;
		    add  = 20480L*65536;
		    ph2 = ph3 = ph5 = ph7 = 0;
#else
		    add = 0x00590000UL; /* 60Hz */
		    add2 = 0x28b0; /* 7kHz */
#endif
		}
	    }
	    if (mode == 3) {
#ifdef AP_IMD
		s_int16 tmp;

		tmp = ((s_int32)ISine(ph2)*16384U +
		       (s_int32)ISine(ph3)*5461U +
		       (s_int32)ISine(ph5)*3277U +
		       (s_int32)ISine(ph7)*2341U)>>14;

		p[0] = p[1] = //tmp;
		    (52428U * (s_int32)tmp + (s_int32)ISine(phs>>16) * 13107) >> 16;
		ph3 += add2*3;
		ph5 += add2*5;
		ph7 += add2*7;
#else
		p[0] = p[1] =
		    ((s_int32)ISine(phs>>16) * 49152U +
		     (s_int32)ISine(ph2)*16384) >> 16;
#endif
		ph2 += add2;
	    } else if (mode == 2) {
		if ((volumeReg & 0xff00U) == 0xfe00U) {
		    p[0] = 0;
		} else {
		    p[0] = ISine(phs>>16);
		}
		if ((volumeReg & 0xff) == 0xfe) {
		    p[1] = 0;
		} else {
		    p[1] = ISine(phs>>16);
		}
	    } else {
		p[0] = p[1] = ISine(phs>>16);
	    }
	    p += 2;
	    phs += add;

	    if (p >= &minifatBuffer[2*16]) {
		p = (s_int16 *)minifatBuffer;
		AudioOutputSamples(p, 16);

		if (mode == 0) {
		    if (add < 0x01000000UL) {
			add += 0x40;
		    } else if (add < 0x10000000UL) {
			add += 0x400;
		    } else {
			add += 0x4000;
		    }
		    if (add >= 0x80000000UL) {
			add = 0;
		    }
		} else if (mode == 1) {
		    wait++;
		    if (wait > 4000) {
			wait = 0;
			if (volumeReg > 0) {
			    volumeReg -= 0x0101;
			    SetVolume();
			}
		    }
		} else if (mode == 2) {
#ifdef AUTO_CROSS
		    wait++;
		    if (wait > (s_int16)(441000/16)) {
			wait = 0;
			volumeReg = (volumeReg<<8)|(volumeReg>>8);
			SetVolume();
		    }
#endif
		}
	    }
	}
    }
}
#endif/*AUDIO_TESTS*/


void main(void) {

    /* Set the leds after nand-boot! */
    /* We set LED2 here too to prevent the display to mess it up later.. */
    PERIP(GPIO1_ODATA) |=  LED1|LED2;      /* POWER led on */
    PERIP(GPIO1_DDR)   |=  LED1|LED2; /* SI and SO to outputs */
    PERIP(GPIO1_MODE)  &= ~(LED1|LED2); /* SI and SO to GPIO */


    player.volumeOffset = -24;
    player.pauseOn = 0;
    //player.randomOn = 0;

#ifdef AUDIO_TESTS
    keyCheck = 0; /* polled key functions */
    if ((ReadGPIO() & 31) == KEY_5) {
	/* Feature button pressed when powerup */
	currentKeyMap = myEmptyMap;
	AudioTests();
    }
#endif

}
