#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vs1000.h>
#include <audio.h>
#include <mappertiny.h>
#include <minifat.h>
#include <codec.h>
#include <vsNand.h>
#include <player.h>
#include <usblowlib.h>

#include <dev1000.h>

#include "bookvars.h"

struct screen {
    u_int16 X, Y, invertMask;
} screen = {
    0, 0, 0
};

#define DISPLAY


#ifdef DISPLAY
void IScreenLocate(register __b0 int x, register __b1 int y); /*34 words*/
void IScreenPutData(register __a1 u_int16 c); /*36 words*/

#define SCREEN_SET_COMMAND_MODE PERIP(GPIO1_CLEAR_MASK) = 0x0004
#define SCREEN_SET_DATA_MODE    PERIP(GPIO1_SET_MASK) = 0x0004
#define SCREEN_SELECT   PERIP(GPIO0_CLEAR_MASK) = 0x4000 //CS2
#define SCREEN_END      PERIP(GPIO0_SET_MASK) = 0x4000 //CS2

// Set x,y pointer of display controller
void IScreenLocate(register __b0 int x, register __b1 int y) { /*34 words*/
  SCREEN_SELECT; //CS low
#ifdef PARALLEL_DISPLAY
  NandPutCommand((screen.Y = y) | 0xb0);
  NandPutCommand((screen.X = x) & 0x0f);
  NandPutCommand((x >> 4) | 0x0010);
#else
  SCREEN_SET_COMMAND_MODE;
  SpiSendReceive((screen.Y = y) | 0xb0);
  SpiSendReceive((screen.X = x) & 0x0f);
  SpiSendReceive((x >> 4) | 0x0010);
#endif
  SCREEN_END;
}


// Write image data octet to display controller
void IScreenPutData(register __a1 u_int16 c) { /*36 words*/
  SCREEN_SELECT; //CS low
#ifdef PARALLEL_DISPLAY
  NandPutAddressOctet(c ^ screen.invertMask);
#else
  SCREEN_SET_DATA_MODE;
  SpiSendReceive(c ^ screen.invertMask);
#endif
  SCREEN_END; //CS high
  if (++screen.X > 131) {
    IScreenLocate(0, ++screen.Y);
  }
}

#endif/*DISPLAY*/



extern struct FsNandPhys fsNandPhys;
extern struct FsPhysical *ph;
extern struct FsMapper *map;
extern struct Codec *cod;
extern struct CodecServices cs;
extern u_int16 codecVorbis[];

#ifdef SAVE_SETTINGS
u_int16 CalcCheck(register __y __i2 u_int16 *p) {
    register int i;
    register u_int16 chk = 0xffffU;
    for (i=0;i<sizeof(settings);i++)
	chk ^= *p++;
    return chk;
}
#endif/*SAVE_SETTINGS*/

void main(void) {
    /* Save settings: */
    clockX = 2;
    SetRate(1000);

    settings.volume = player.volume;
#ifdef SAVE_SETTINGS
    if (CalcCheck((__y void *)&settings)) {
	int n, erased;
	__y u_int32 block;
#if 0
	puts("Saving settings");
#endif
 again:
	n = ph->eraseBlockSize;
	erased = 0;
	if (n < 256) {
	    block = 256-n; /* use the last erase block before mapper */
	} else {
	    /*ph->eraseBlocks--;*/ /* already done at startup */
	    block = (u_int32)ph->eraseBlocks * n; /* use the last erase block*/
	}

//ph->Erase(ph, block);
	while (n--) {
	    int i = 0, touched = 0;
#if 0
	    puthex(block);
	    puts("=block");
#endif
	    ph->Read(ph, block, 1, minifatBuffer, NULL);
	    while (i < 256) {
		register u_int16 and = 0xffffU, or = 0;
		{
		    register int j;
		    register u_int16 *p = minifatBuffer+i;
		    for (j=0;j<sizeof(settings);j++) {
			and &= *p;
			or  |= *p;
			p++;
		    }
		}
		if (or == 0) {
#if 0
		    puts("Clear");
#endif
		    /* cleared entry, nothing to do */
		} else if (and == 0xffffU) {
		    /* free entry */
#if 0
		    puthex(block);
		    puthex(i);
		    puts("=wrote");
#endif
		    settings.check = 0;
		    settings.check = CalcCheck((__y void *)&settings);
		    memcpyYX(minifatBuffer+i, &settings, sizeof(settings));
		    touched = 2;
		    break;
		} else {
#if 0
		    puts("Dirty");
#endif
		    memset(minifatBuffer+i, 0, sizeof(settings));
		    touched = 1;
		}
		i += sizeof(settings);
	    }
	    if (touched) {
		ph->Write(ph, block, 1, minifatBuffer, NULL);
		if (touched == 2) {
		    break; /* Done.. */
		}
	    }
	    block++;
	    if (n == 0 && erased == 0) {
		/* Was last chance.. Erase and start over. */
		n = ph->eraseBlockSize;
		if (n < 256) {
		    block = 256-n; /* use the last erase block before mapper */
		} else {
		    block = (u_int32)ph->eraseBlocks * n;
		}
		ph->Erase(ph, block);
		erased = 1;
	    }
	}
    } else {
#if 0
	puts("Settings ok");
#endif
    }
#endif

    PERIP(GPIO1_ODATA) &= ~LED2; /* backlight off */
    SCREEN_SELECT; //CS low
#ifdef PARALLEL_DISPLAY
    NandPutCommand(0xae); /*display off*/
#else
    SCREEN_SET_COMMAND_MODE;
    SpiSendReceive(0xae);
    BusyWait10();
    SpiSendReceive(0xad); //dcdc off
    SpiSendReceive(0x8a); //dcdc off value
    BusyWait10();
#endif
    SCREEN_END;
    RealPowerOff();
}
