#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vs1000.h>
#include <audio.h>
#include <mappertiny.h>
#include <minifat.h>
#include <codec.h>
#include <vsNand.h>
#include <player.h>
#include <usblowlib.h>

#include <dev1000.h>

#include "bookvars.h"

struct screen {
    u_int16 X, Y, invertMask;
} screen = {
    0, 0, 0
};

extern struct {
    u_int16 level;
    u_int32 checkTime;
} battery;

volatile u_int16 menuMode = 0;

#define USE_DEBUG


#define USE_USB
#ifdef MAPPERLESS
#undef USE_USB
#endif
#define DISPLAY
#define BATTERY_LEVEL
//#define AUTO_POWEROFF //disabled to allow easier USB-programming
#define POWERON_KEYWAIT 4


#ifdef DISPLAY
void ScreenLocate(register __b0 int x, register __b1 int y); /*34 words*/
void ScreenPutData(register __a1 u_int16 c); /*36 words*/

#define SCREEN_SELECT   PERIP(GPIO0_CLEAR_MASK) = 0x4000 //CS2
#define SCREEN_END      PERIP(GPIO0_SET_MASK) = 0x4000 //CS2

#ifdef USE_DEBUG
void ScreenStatus(void) {
    u_int16 tmp;
    SCREEN_SELECT;
    NandGetOctets(1, &tmp);
    SCREEN_END;
    puthex(tmp);
    puts("=status"); //0x00 = display on
}
#endif

// Set x,y pointer of display controller
void ScreenLocate(register __b0 int x, register __b1 int y) { /*34 words*/
  SCREEN_SELECT; //CS low
  NandPutCommand((screen.Y = y) | 0xb0);
  screen.X = x;
  NandPutCommand(screen.X & 0x0f);
  NandPutCommand((x >> 4) | 0x0010);
  SCREEN_END;
}


// Write image data octet to display controller
void ScreenPutData(register __a1 u_int16 c) { /*36 words*/
  SCREEN_SELECT; //CS low
  NandPutAddressOctet(c ^ screen.invertMask);
  SCREEN_END; //CS high
  if (++screen.X > 131) {
      ScreenLocate(0, ++screen.Y);
  }
}
void ScreenClear() { /*29 words*/
    register u_int16 i;
    ScreenLocate(0, 0);
    for (i=0;i<8*132;i++)
	ScreenPutData(0); /*ScreenPutData() handles the line wraps*/
    ScreenLocate(0, 0);
}


#if 0
auto void ScreenPutGfx(register __i1 const u_int16 *d,
		       register __c0 u_int16 words);
#else
auto void ScreenPutGfx(register __i1 const u_int16 *d,
		       register __c0 u_int16 words) { /*29 words*/
    register int i;
    for (i=0;i<words;i++) {
	ScreenPutData((s_int16)*d>>8);
	ScreenPutData(*d++);
    }
}
#endif

#endif/*DISPLAY*/



extern struct FsNandPhys fsNandPhys;
extern struct FsPhysical *ph;
extern struct FsMapper *map;
extern struct Codec *cod;
extern struct CodecServices cs;
extern u_int16 codecVorbis[];

const u_int16 latin1[] = {
	0x0000,0x0000,0x7e7e, /*0x00 ? */
	0x7e7e,0x7e7e,0x7e7e, /*0x01 ? */
	0x7e7e,0x7e42,0x4242, /*0x02 ? */
	0x4242,0x4242,0x4242, /*0x03 ? */
	0x7e7e,0x1818,0x0000, /*0x04 ? */
	0x007f,0x3e1c,0x0800, /*0x05 ? */
	0x7f7f,0x0000,0x7f7f, /*0x06 ? */
	0x7f7f,0x6363,0x7f7f, /*0x07 ? */
	0x1038,0x5410,0x1010, /*0x08 ? */
	0x1818,0x1818,0x7eff, /*0x09 ? */
	0xff7e,0x1818,0x1818, /*0x0a ? */
	0x0000,0xf0f8,0x1818, /*0x0b ? */
	0x1818,0xf8f0,0x0000, /*0x0c ? */
	0x0000,0xffff,0x1818, /*0x0d ? */
	0x1818,0xffff,0x0000, /*0x0e ? */
	0x1818,0xffff,0x1818, /*0x0f ? */
	0x0000,0x0000,0x0000, /*0x10 ? */
	0x1100,0x0011,0x0000, /*0x11 ? */
	0x1100,0x4411,0x0044, /*0x12 ? */
	0x5500,0x5500,0x5500, /*0x13 ? */
	0x55aa,0x55aa,0x55aa, /*0x14 ? */
	0x77aa,0x55aa,0x77aa, /*0x15 ? */
	0x77aa,0xddaa,0x77aa, /*0x16 ? */
	0xffaa,0xffaa,0xffaa, /*0x17 ? */
	0xffbb,0xffee,0xffbb, /*0x18 ? */
	0xffbb,0xffff,0xffbb, /*0x19 ? */
	0xffff,0xffff,0xffff, /*0x1a ? */
	0x0000,0x0f1f,0x1818, /*0x1b ? */
	0x1818,0x1f0f,0x0000, /*0x1c ? */
	0x1818,0x1f1f,0x1818, /*0x1d ? */
	0x1818,0xf8f8,0x1818, /*0x1e ? */
	0x0000,0xf0f8,0x187e, /*0x1f ? */
	0x0000,0x0000,0x0000, /*0x20   */
	0x0000,0x5f5f,0x0000, /*0x21 ! */
	0x0707,0x0007,0x0700, /*0x22 " */
	0x147f,0x1414,0x7f14, /*0x23 # */
	0x042a,0x7f2a,0x3a10, /*0x24 $ */
	0x4626,0x1008,0x6462, /*0x25 % */
	0x344a,0x4a54,0x2050, /*0x26 & */
	0x0004,0x0703,0x0000, /*0x27 ' */
	0x001c,0x3e63,0x4100, /*0x28 ( */
	0x0041,0x633e,0x1c00, /*0x29 ) */
	0x082a,0x1c1c,0x2a08, /*0x2a * */
	0x0808,0x3e3e,0x0808, /*0x2b + */
	0x0080,0xe060,0x0000, /*0x2c , */
	0x0808,0x0808,0x0808, /*0x2d - */
	0x0000,0x6060,0x0000, /*0x2e . */
	0x6030,0x180c,0x0603, /*0x2f / */
	0x3e7f,0x5145,0x7f3e, /*0x30 0 */
	0x0042,0x7f7f,0x4000, /*0x31 1 */
	0x727b,0x4949,0x6f66, /*0x32 2 */
	0x226b,0x4949,0x7f36, /*0x33 3 */
	0x0f0f,0x0808,0x7f7f, /*0x34 4 */
	0x2f6f,0x4949,0x7b33, /*0x35 5 */
	0x3e7f,0x4949,0x7b32, /*0x36 6 */
	0x0101,0x797d,0x0703, /*0x37 7 */
	0x367f,0x4949,0x7f36, /*0x38 8 */
	0x266f,0x4949,0x7f3e, /*0x39 9 */
	0x0000,0x3636,0x0000, /*0x3a : */
	0x0040,0x7636,0x0000, /*0x3b ; */
	0x081c,0x3663,0x4100, /*0x3c < */
	0x1414,0x1414,0x1414, /*0x3d = */
	0x0041,0x6336,0x1c08, /*0x3e > */
	0x0203,0x5159,0x0f06, /*0x3f ? */
	0x3e41,0x4d53,0x4f0e, /*0x40 @ */
	0x7e7f,0x0909,0x7f7e, /*0x41 A */
	0x7f7f,0x4949,0x7f36, /*0x42 B */
	0x3e7f,0x4141,0x6322, /*0x43 C */
	0x7f7f,0x4141,0x7f3e, /*0x44 D */
	0x7f7f,0x4949,0x4941, /*0x45 E */
	0x7f7f,0x0909,0x0101, /*0x46 F */
	0x3e7f,0x4149,0x7b3a, /*0x47 G */
	0x7f7f,0x0808,0x7f7f, /*0x48 H */
	0x0041,0x7f7f,0x4100, /*0x49 I */
	0x2060,0x4040,0x7f3f, /*0x4a J */
	0x7f7f,0x1c36,0x6341, /*0x4b K */
	0x7f7f,0x4040,0x4040, /*0x4c L */
	0x7f7e,0x0c0c,0x7e7f, /*0x4d M */
	0x7f7e,0x0c18,0x3f7f, /*0x4e N */
	0x3e7f,0x4141,0x7f3e, /*0x4f O */
	0x7f7f,0x0909,0x0f06, /*0x50 P */
	0x3e7f,0x41c1,0xffbe, /*0x51 Q */
	0x7f7f,0x1939,0x6f46, /*0x52 R */
	0x266f,0x4949,0x7b32, /*0x53 S */
	0x0101,0x7f7f,0x0101, /*0x54 T */
	0x3f7f,0x4040,0x7f3f, /*0x55 U */
	0x1f3f,0x6060,0x3f1f, /*0x56 V */
	0x7f3f,0x1818,0x3f7f, /*0x57 W */
	0x6377,0x1c1c,0x7763, /*0x58 X */
	0x070f,0x7878,0x0f07, /*0x59 Y */
	0x6171,0x594d,0x4743, /*0x5a Z */
	0x007f,0x7f41,0x4141, /*0x5b [ */
	0x0306,0x0c18,0x3060, /*0x5c \ */
	0x0041,0x4141,0x7f7f, /*0x5d ] */
	0x0406,0x0303,0x0604, /*0x5e ^ */
	0x8080,0x8080,0x8080, /*0x5f _ */
	0x0001,0x0302,0x0400, /*0x60 ` */
	0x2074,0x5454,0x7c78, /*0x61 a */
	0x7f7f,0x4444,0x7c38, /*0x62 b */
	0x387c,0x4444,0x6c28, /*0x63 c */
	0x387c,0x4444,0x7f7f, /*0x64 d */
	0x387c,0x5454,0x5c18, /*0x65 e */
	0x447e,0x7f45,0x4501, /*0x66 f */
	0x18bc,0xa4a4,0xfc78, /*0x67 g */
	0x7f7f,0x0404,0x7c78, /*0x68 h */
	0x0044,0x7d7d,0x4000, /*0x69 i */
	0x40c0,0x84fd,0x7d00, /*0x6a j */
	0x7f7f,0x183c,0x6440, /*0x6b k */
	0x0041,0x7f7f,0x4000, /*0x6c l */
	0x7c7c,0x0438,0x0478, /*0x6d m */
	0x7c7c,0x0404,0x7c78, /*0x6e n */
	0x387c,0x4444,0x7c38, /*0x6f o */
	0xfcfc,0x2424,0x3c18, /*0x70 p */
	0x183c,0x2424,0xfcfc, /*0x71 q */
	0x7c7c,0x0404,0x0c08, /*0x72 r */
	0x485c,0x5454,0x7420, /*0x73 s */
	0x043f,0x7f44,0x6420, /*0x74 t */
	0x3c7c,0x4040,0x7c7c, /*0x75 u */
	0x1c3c,0x6060,0x3c1c, /*0x76 v */
	0x3c7c,0x4078,0x403c, /*0x77 w */
	0x446c,0x3838,0x6c44, /*0x78 x */
	0x1cbc,0xa0a0,0xfc7c, /*0x79 y */
	0x4464,0x745c,0x4c44, /*0x7a z */
	0x107c,0xee82,0x8200, /*0x7b { */
	0x0000,0xffff,0x0000, /*0x7c | */
	0x0082,0x82ee,0x7c10, /*0x7d } */
	0x1008,0x0810,0x1008, /*0x7e ~ */
	0x1010,0x1054,0x3810, /*0x7f  */
	0x143e,0x5555,0x4122, /*0x80 ? */
	0x0000,0x0000,0x00ff, /*0x81 ? */
	0x0000,0x8040,0x0000, /*0x82 ? */
	0x90f8,0x7c14,0x1404, /*0x83 ? */
	0x0080,0x4080,0x4000, /*0x84 ? */
	0x4000,0x4000,0x4000, /*0x85 ? */
	0x0808,0xfe08,0x0800, /*0x86 ? */
	0x2828,0xfe28,0x2800, /*0x87 ? */
	0x0002,0x0102,0x0000, /*0x88 ? */
	0x2616,0x6864,0x0260, /*0x89 ? */
	0x485c,0x5556,0x7524, /*0x8a ? */
	0x0010,0x2844,0x0000, /*0x8b ? */
	0x387c,0x4438,0x7c54, /*0x8c ? */
	0x0804,0x7e04,0x0800, /*0x8d ? */
	0x4465,0x765d,0x4c44, /*0x8e ? */
	0x1c14,0x1c22,0x417f, /*0x8f ? */
	0x7f41,0x221c,0x141c, /*0x90 ? */
	0x0001,0x0200,0x0000, /*0x91 ? */
	0x0002,0x0100,0x0000, /*0x92 ? */
	0x0102,0x0001,0x0200, /*0x93 ? */
	0x0201,0x0002,0x0100, /*0x94 ? */
	0x1c3e,0x3e3e,0x1c00, /*0x95 ? */
	0x0008,0x0808,0x0000, /*0x96 ? */
	0x1818,0x1818,0x1818, /*0x97 ? */
	0x0201,0x0102,0x0201, /*0x98 ? */
	0x0107,0x0107,0x0307, /*0x99 ? */
	0x085c,0x5556,0x7520, /*0x9a ? */
	0x0044,0x2810,0x0000, /*0x9b ? */
	0x387c,0x4438,0x5458, /*0x9c ? */
	0x1020,0x7f20,0x1000, /*0x9d ? */
	0x4869,0x6a59,0x5848, /*0x9e ? */
	0x040d,0x7878,0x0d04, /*0x9f ? */
	0xff00,0x0000,0x0000, /*0xa0  */
	0x0000,0xfafa,0x0000, /*0xa1  */
	0x1824,0x247e,0x2424, /*0xa2  */
	0x507c,0x5252,0x4420, /*0xa3  */
	0x2d12,0x1212,0x2d00, /*0xa4  */
	0x1556,0x7c56,0x1500, /*0xa5  */
	0x0000,0xe700,0x0000, /*0xa6  */
	0x4e53,0x6539,0x0000, /*0xa7  */
	0x0101,0x0001,0x0100, /*0xa8  */
	0x3e6b,0x5555,0x633e, /*0xa9  */
	0x1018,0x1515,0x1e10, /*0xaa  */
	0x0814,0x2a55,0x2241, /*0xab  */
	0x0808,0x0808,0x0838, /*0xac  */
	0x0808,0x0808,0x0808, /*0xad  */
	0x3e63,0x5d45,0x633e, /*0xae  */
	0x0101,0x0101,0x0101, /*0xaf  */
	0x0609,0x090f,0x0600, /*0xb0  */
	0x2424,0x2e24,0x2400, /*0xb1  */
	0x1219,0x1d17,0x1200, /*0xb2  */
	0x0a11,0x151f,0x0a00, /*0xb3  */
	0x0004,0x0203,0x0100, /*0xb4  */
	0xfcfc,0x2024,0x3c1c, /*0xb5  */
	0x064f,0x497f,0x017f, /*0xb6  */
	0x0000,0x1818,0x0000, /*0xb7  */
	0x0080,0xa0e0,0xc000, /*0xb8  */
	0x0012,0x1f1f,0x1000, /*0xb9  */
	0x1619,0x1919,0x1600, /*0xba  */
	0x4122,0x552a,0x1408, /*0xbb  */
	0x1708,0x043a,0x21f8, /*0xbc  */
	0x1708,0xc4aa,0xa998, /*0xbd  */
	0x553f,0x1038,0x24f2, /*0xbe  */
	0x60f0,0x9a8a,0xc040, /*0xbf  */
	0x787c,0x1516,0x7c78, /*0xc0  */
	0x787c,0x1615,0x7c78, /*0xc1  */
	0x707a,0x292a,0x7870, /*0xc2  */
	0x7279,0x292a,0x7a71, /*0xc3  */
	0x7d7f,0x0a0a,0x7f7d, /*0xc4  */
	0x787e,0x1515,0x7e78, /*0xc5  */
	0x7c7e,0x0a7e,0x7e4a, /*0xc6  */
	0x1c3e,0xa262,0x3614, /*0xc7  */
	0x7c7c,0x5556,0x5444, /*0xc8  */
	0x7c7c,0x5655,0x5444, /*0xc9  */
	0x7c7e,0x5556,0x5444, /*0xca  */
	0x7d7d,0x5454,0x5545, /*0xcb  */
	0x0045,0x7e7c,0x4400, /*0xcc  */
	0x0044,0x7c7e,0x4500, /*0xcd  */
	0x0046,0x7d7e,0x4400, /*0xce  */
	0x0145,0x7c7c,0x4501, /*0xcf  */
	0x7c7c,0x5444,0x7c38, /*0xd0  */
	0x7e7d,0x0912,0x3e7d, /*0xd1  */
	0x387c,0x4546,0x7c38, /*0xd2  */
	0x387c,0x4645,0x7c38, /*0xd3  */
	0x387e,0x4546,0x7c38, /*0xd4  */
	0x3a7d,0x4546,0x7e39, /*0xd5  */
	0x397d,0x4444,0x7d39, /*0xd6  */
	0x4428,0x1028,0x4400, /*0xd7  */
	0x5c3e,0x524a,0x7c3a, /*0xd8  */
	0x3c7c,0x4142,0x7c3c, /*0xd9  */
	0x3c7c,0x4241,0x7c3c, /*0xda  */
	0x3c7e,0x4142,0x7c3c, /*0xdb  */
	0x3d7d,0x4040,0x7d3d, /*0xdc  */
	0x044c,0x7271,0x4c04, /*0xdd  */
	0x7f7f,0x5514,0x1c08, /*0xde  */
	0xfeff,0x014b,0x7e34, /*0xdf  */
	0x2074,0x5556,0x7c78, /*0xe0  */
	0x2074,0x5655,0x7c78, /*0xe1  */
	0x2076,0x5556,0x7c78, /*0xe2  */
	0x2275,0x5556,0x7e79, /*0xe3  */
	0x2175,0x5454,0x7d79, /*0xe4  */
	0x2074,0x5555,0x7c78, /*0xe5  */
	0x2054,0x5438,0x5458, /*0xe6  */
	0x183c,0xa464,0x2c28, /*0xe7  */
	0x387c,0x5556,0x5c08, /*0xe8  */
	0x387c,0x5655,0x5c08, /*0xe9  */
	0x387e,0x5556,0x5c08, /*0xea  */
	0x397d,0x5454,0x5d09, /*0xeb  */
	0x0049,0x7a78,0x4000, /*0xec  */
	0x0048,0x7a79,0x4000, /*0xed  */
	0x004a,0x797a,0x4000, /*0xee  */
	0x0145,0x7c7c,0x4101, /*0xef  */
	0x2070,0x5456,0x3c04, /*0xf0  */
	0x7a79,0x090a,0x7a71, /*0xf1  */
	0x3078,0x494a,0x7830, /*0xf2  */
	0x3078,0x4a49,0x7830, /*0xf3  */
	0x307a,0x494a,0x7830, /*0xf4  */
	0x3279,0x494a,0x7a31, /*0xf5  */
	0x327a,0x4848,0x7a32, /*0xf6  */
	0x1010,0x5454,0x1010, /*0xf7  */
	0x582c,0x745c,0x6834, /*0xf8  */
	0x3878,0x4142,0x7878, /*0xf9  */
	0x3878,0x4241,0x7878, /*0xfa  */
	0x387a,0x4142,0x7878, /*0xfb  */
	0x3a7a,0x4040,0x7a7a, /*0xfc  */
	0x18b8,0xa2a1,0xf878, /*0xfd  */
	0xffff,0x2828,0x3810, /*0xfe  */
	0x1aba,0xa0a0,0xfa7a
};


void main(void) { /*264 words*/
    register u_int32 time;

    clockX = 6; /*3.0x*/
    /* set cs.sampleRate to non-zero because we use cs.Output() */
    cs.sampleRate = 44100U;

    /* Turn off after 5 minutes (300secs) in pause mode. */
    player.offDelay = 300/5; 
    //player.offDelay = 20/5; 

    {
	/* The display needs the IO voltage to be about 2.6V */
	static const __y myVoltages[] = {
	    /* Core I/O Analog */
	    22, 27, 12, /* 2.2 3.3 2.8 player */
	    27, 27, 30, /* 2.5 3.3 3.6 USB */
	    14, 17,  0, /* 1.8 2.6 3.6 (USB) suspend  18=2.65*/
	    22, 27, 12  /* 2.2 3.3 2.8 user -- used for battery measurement*/
	};
	memcpyYX(voltages, myVoltages, sizeof(myVoltages));
	/* battery measurement does this.. */
	//PowerSetVoltages(&voltages[voltCorePlayer]);
    }

    /* Set the leds after nand-boot! */
    PERIP(GPIO1_ODATA) |=  LED1|LED2;   /* POWER led on */
    PERIP(GPIO1_DDR)   |=  LED1|LED2;   /* SI and SO to outputs */
    PERIP(GPIO1_MODE)  &= ~(LED1|LED2); /* SI and SO to GPIO */


    player.volumeOffset = -24;
    player.pauseOn = 0;
    //player.randomOn = 0;
    keyOld = KEY_POWER;
    keyOldTime = -32767; /* ignores the first release of KEY_POWER */

    PlayerVolume();

#ifdef DISPLAY
    /* Wait until XCS high (display /RES) */
#if 0
    do {
	Sleep();
    } while ((PERIP(GPIO1_IDATA) & 1) == 0);
#else
    PERIP(GPIO1_ODATA) &= ~1;   /* XCS down */
    PERIP(GPIO1_DDR)   |=  1;   /* XCS to output */
    PERIP(GPIO1_MODE)  &= ~1;   /* XCS to GPIO */
    BusyWait10();
    PERIP(GPIO1_ODATA) |=  1;   /* XCS on */
#endif
    PERIP(GPIO0_ODATA) |= 0x4000; //CS2 high
    PERIP(GPIO0_DDR)   |= 0x4000; //CS2 is Output
    PERIP(GPIO0_MODE)  &= ~0x4000; //CS2 is GPIO (Display Chip Select)

    do {
	Sleep();
    } while ((PERIP(GPIO1_IDATA) & 1) == 0);
#ifdef USE_DEBUG
    PERIP(NFLSH_CTRL) |= 31; /* max waits */
    {
	register int i;
	for (i=0;i<4;i++) {
	    ScreenStatus();
	}
    }
    if (0)
    {
	int i;
	PERIP(GPIO0_MODE) = 0;
	PERIP(GPIO0_DDR)  = ~(0x4000|(1<<10)|(1<<8));//0xffff;
	PERIP(GPIO1_MODE) = 0x30; /*keep TX/RX*/
	for (i=0;i<20;i++) {
	    if (i < 16)
		PERIP(GPIO0_ODATA) = (1<<i);
	    else
		PERIP(GPIO1_ODATA) = (1<<(i-16));

	    puthex(PERIP(GPIO0_MODE));
	    puthex(PERIP(GPIO0_DDR));
	    puthex(PERIP(GPIO0_ODATA));
	    puthex(PERIP(GPIO0_IDATA));
	    puts("=GPIO0 MODE,DDR,ODATA,IDATA");
	    puthex(PERIP(GPIO1_MODE));
	    puthex(PERIP(GPIO1_DDR));
	    puthex(PERIP(GPIO1_ODATA));
	    puthex(PERIP(GPIO1_IDATA));
	    puts("=GPIO1 MODE,DDR,ODATA,IDATA");
	}
    }
#endif
    SCREEN_SELECT; //CS low
    {
	static const u_int16 init[] = {
	    0xaf, // Display ON
	    0x2f, // Booster, Regulator, Follower ON
	    0xa4, // Normal display (0xa5=All pixels on)
	    0xc8, /* COM scan direction: reverse */
	    0xa1, /* SEG output correspondance: reverse */
	    0
	};
	register const u_int16 *p = init;
	while (*p) {
	    NandPutCommand(*p++);
	}
    }
    SCREEN_END;
#ifdef USE_DEBUG
    {
	register int i;
	for (i=0;i<4;i++) {
	    ScreenStatus();
	}
    }
#endif
#if 0 //def USE_DEBUG
    {
	register int i;
	u_int16 tmp;
	for (i=0;i<8*128/2;i++) {
	    if ((i & 63) == 0)
		ScreenLocate(0, i/64);
	    BusyWait10();
	    SCREEN_SELECT;
	    PERIP(GPIO0_ODATA) |= GPIO0_ALE; // pull NFlash ALE pin high
	    NandGetOctets(2, &tmp);
	    PERIP(GPIO0_ODATA) &= ~GPIO0_ALE; // pull NFlash ALE pin low 
	    SCREEN_END;
	    puthex(tmp);
	}
	puts("=data");
    }
#endif
    while (1) {
	unsigned char tmp[160];
	int i = 0;
	fputs("string> ", stdout);
	fgets(tmp, 160, stdin);
	ScreenLocate(0, 0);
//	screen.invertMask = 0x80;
	while (tmp[i]) {
	    u_int16 *p = &latin1[tmp[i]*3];//-32*3;
	    ScreenPutGfx(p, 3);
	    ScreenPutData(0);
	    i++;
	    if (screen.X > 132-6) {
		ScreenLocate(0, screen.Y+1);
	    }
	}
    }
#endif

}
